/*
  This is a prototype robot for experimenting with PWM steering matched to a servo controlled front wheel.
 This is a fairly stupid way to build a robot, but it will be educational to try it.
 More info can be found at:
 http://letsmakerobots.com/node/25644
 This version was written for the uBotino from Ro-Bot-X (http://robotxdesigns.ca)
 by ignoblegnome: http://letsmakerobots.com/user/5293
 
 uBotino board pinout:
 
 uBotino Funct Arduino  ATmega168      Arduino Funct uBotino
 +-----\/----+
 Reset       1| PC6   PC5 |28  D19 A5  SCL  RightBumper
 Rx    D0    2| PD0   PC4 |27  D18 A4  SDA  LeftBumper
 Tx    D1    3| PD1   PC3 |26  D17 A3        
 Int0  D2    4| PD2   PC2 |25  D16 A2        
 Int1  D3    5| PD3   PC1 |24  D15 A1        IR sensor
 Spk       D4    6| PD4   PC0 |23  D14 A0        Ping
 7| VCC   GND |22  
 8| GND  AREF |21  
 Xtal        9| PB6  AVCC |20  
 Xtal       10| PB7   PB5 |19  D13      SCK  LED      
 M1A OC0B  D5   11| PD5   PB4 |18  D12      MISO 
 M2A OC0A  D6   12| PD6   PB3 |17  D11 OC2A MOSI       
 M2B       D7   13| PD7   PB2 |16  D10 OC1B      
 M1B       D8   14| PB0   PB1 |15  D 9 OC1A      Pan servo
 +-----------+
 
 Version 0.5: Working on new navigation techniques
 Version 0.4: Added 'SoundnFury' module and some flash code
 Version 0.3: Added Sharp sensor and basic obstacle avoidance using 1/3 rate turns
 Version 0.2: Added steering servo
 Version 0.1: Basic motor testing
 */

#include <Servo.h>

#define DEBUG 0    // set to 1 to print to serial monitor, 0 to disable

#define Motor_1_PWM   5 // digital pin 5    // Right Motor
#define Motor_1_Dir   8 // digital pin 8
#define Motor_2_PWM   6 // digital pin 6   // Left Motor
#define Motor_2_Dir   7 // digital pin 7

#define sharppin  14     // analog pin for Sharp IR (analog pin 0 for uBotino is 14, just like the Uno or Duemilanove
#define turnservopin 13  // digital pin 13

// SoundnFury stuff
#define LED1_Red  2  // digital pin 2
#define LED1_Green 3 // digital pin 3 (pwm)
#define LED2_Red  9  // digital pin 9 (pwm)
#define LED2_Green 1 // digital pin 1
#define LED3_Red  12 // digital pin 12
#define LED3_Green  11 // digital pin 11 (pwm)
#define LED4_Red  10  // digital pin 10 (pwm)
#define LED4_Green  0  // digital pin 0
#define piezo 4          // digital pin 4

// Servo angle values are measured from 0 degrees on the left to 180 degrees on the right.
// Variable names reflect angle required for each turn mode.
// Servo actually swings + or - of 180, so each servo requires it's own compensation factor.
// The servo value for each theoretical angle must be determined, due to non-linear servo response.

// for driving straight forward or back
#define servocenter 90   // 

// for turning with opposite wheels at full speed
#define servo0 14  // 0 degrees, compensated for servo
#define servo180 166  // 180 degrees, compenasted for servo

// for turning with inner wheel stopped
#define servo30 39  // 30 degrees, compensated for servo
#define servo150 139  // 150 degrees, compensated for servo

// for turning with inner wheel at 1/3 rate of outer wheel
#define servo49 54  // 49 degrees, compensated for servo
#define servo131 120  // 131 degrees, compensated for servo

// for turning with inner wheel at 2/3 rate of outer wheel
#define servo71 76  // 71 degrees, compensated for servo
#define servo109 104 // 109 degrees, compensated for servo

Servo turnservo; // creates head servo object

// Constants
//

int dangerlevel = 200;   // how far away should things be, before we react?
int softlevel = 300;    // how far away to trigger a soft turn?
int hardlevel = 400;    // how far away to trigger a hard turn?
int pivotlevel = 500;    // how far away to trigger a pivot turn?
int backoff = 600;       // if you are this close, back up a bit
int turn = 900;          // this sets how much we should turn
int servo_turn = 700;    // this sets how long we should wait for the servo to turn before we measure distance

// Variables

// Navigation stuff
int currDist = 0;    // holds the current distance
int compDist = 0;  // holds a distance to compare currDist to
int direction = 0;  // 0 = left, 1 = right

byte dir=0;
byte speed1=200;  // 255 max; adjust if needed to keep robot going straight
byte speed2=200;  // 255 max; adjust if needed to keep robot going straight
int turn90=500;
int turn45=250;
int stopTime=200;
int goTime=5000;
int USdistance=0;
int treshold=20; //20cm min distance

// SoundnFury stuff
unsigned int toggle;
unsigned int fade;
unsigned int LED1_pwm = 255;
unsigned int LED2_pwm = 255;
unsigned int LED3_pwm = 255;
unsigned int LED4_pwm = 255;
unsigned int LED1_old = 0;
unsigned int LED2_old = 0;
unsigned int LED3_old = 0;
unsigned int LED4_old = 0;

int LED1_Red_state = LOW;
int LED1_Green_state = LOW; 
int LED2_Red_state = LOW;
int LED2_Green_state = LOW;
int LED3_Red_state = LOW;
int LED3_Green_state = LOW;
int LED4_Red_state = LOW;
int LED4_Green_state = LOW;

// Timing stuff
long previousMillis = 0;  // store the last time Millis was recorded
long interval = 1000;       // interval for timed action

void setup() {

  // Set for Output and Input pins
  for (int pinindex = 0; pinindex < 14; pinindex++) {
    pinMode(pinindex, OUTPUT);  // set pins 0 to 13 as outputs
  }

  for (int pinindex = 15; pinindex < 20; pinindex++) {
    digitalWrite(pinindex, HIGH);  // set all the internal pullups for unused analog pins
  }

  // set motor pins LOW so the motors are breaked
  totalhalt();

  pinMode(sharppin, INPUT); // uses an analog pin so it has to be declared as digital in

  turnservo.attach(turnservopin);
  turnservo.write(servocenter); //90
  delay(20);

  // Initialize random seed for flashy LED effects and random turn direction
  randomSeed(analogRead(0));  // seed the random function before setting pullup resistors
  direction = random(2);      // pick either 0 (left) or 1 (right) for initial direction

  // Debug only if DEBUG is set to 1
  if(DEBUG){
    Serial.begin (19200);
    Serial.println("start");
  }
}

void loop() {
  // Timing stuff
  unsigned long currentMillis = millis();

  // Stuff for debugging. Only active if DEBUG is set to 1 above 
  Serial.print("Current Distance: ");
  Serial.println(currDist);

  // Navigation stuff
  currDist = analogRead(sharppin);
  if(currDist < dangerlevel) {  // lower reading from sharp is greater distance
    forward();      // if no danger, drive ahead
  }
  else if(currDist < softlevel) { 
    body_softturn();
  }
  else if(currDist < hardlevel) {
    body_hardturn();
  }
  else if(currDist < pivotlevel) {
    body_pivot();
  }
  else if(currDist > backoff) {
    backup();        // you're too close! back up
    direction = random(2);      // pick either 0 (left) or 1 (right) for initial direction
  }
  else {              // I don't think this will happen; consider eliminating
    whichway();      // if obstacle ahead then decide with way is better
  }
  
      Serial.print("Distance: ");
      Serial.println("currDist");
  
  delay(250);

  // SoundnFury stuff

  if(currentMillis - previousMillis > interval) {
    // save the last time you blinked the LED 
    previousMillis = currentMillis;   

    // if the LED is off turn it on and vice-versa:
    if (LED1_Red_state == LOW) {
      LED1_Red_state = HIGH;
      LED1_Green_state = LOW;
      LED2_Red_state = LOW;
      LED2_Green_state = HIGH;
    }
    else {
      LED1_Red_state = LOW;
      LED1_Green_state = HIGH;
      LED2_Red_state = HIGH;
      LED2_Green_state = LOW;
    }
    // set the LED with the ledState of the variable:
    digitalWrite(LED1_Red, LED1_Red_state);
    digitalWrite(LED1_Green, LED1_Green_state);
    digitalWrite(LED2_Red, LED2_Red_state);
    digitalWrite(LED2_Green, LED2_Green_state);
  }
}

void whichway() {
  totalhalt();    // first stop!

  turnservo.write(servo131);        // look right
  delay(servo_turn);  // wait for servo to stop turning
  int rDist = analogRead(sharppin);   // check distance

  turnservo.write(servo49);        // look left
  delay(servo_turn);  // wait for servo to stop turning
  int lDist = analogRead(sharppin);   // check distance

  if(lDist < rDist) {  // lower analog value is further distance 
    direction = 0;    // set the direction to left
    body_spin();  // spin left
    delay(turn90);
  }
  else {
    direction = 1;      // set the direction to right
    body_spin();
    delay(turn90);
  }
  return;
}  

void forward() {      // drive forward
  turnservo.write(servocenter);
  digitalWrite(Motor_1_Dir, LOW); // forward
  digitalWrite(Motor_2_Dir, LOW); // forward
  analogWrite(Motor_1_PWM, speed1); // 
  analogWrite(Motor_2_PWM, speed2); //
  return;
}  

void backup() {
  turnservo.write(servocenter);
  digitalWrite(Motor_1_Dir, HIGH); // reverse
  digitalWrite(Motor_2_Dir, HIGH); // reverse
  analogWrite(Motor_1_PWM, 255-speed1); // 
  analogWrite(Motor_2_PWM, 255-speed2); //
  return;
}  

void body_spin() {      // left and right wheels spin in opposite direction at same speed
  switch (direction) {
  case 0:  // spin left
    turnservo.write(servo0);        // turn left
    delay(servo_turn);  // wait for servo to stop turning
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, HIGH); // reverse
    analogWrite(Motor_1_PWM, speed1); // 
    analogWrite(Motor_2_PWM, 255-speed2); //
    break;
  case 1:  // spin right
    turnservo.write(servo180);        // turn right
    delay(servo_turn);  // wait for servo to stop turning
    digitalWrite(Motor_1_Dir, HIGH); // reverse
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, 255-speed1); // 
    analogWrite(Motor_2_PWM, speed2); //
    break;
  }  
}  

void body_pivot() {    // inner wheel is stopped while outer wheel is a full speed
  switch (direction) {
  case 0:  // turn left
    turnservo.write(servo30);        // turn left
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, speed1); // 
    analogWrite(Motor_2_PWM, 0); //
    break;
  case 1:  // turn right
    turnservo.write(servo150);        // turn right
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, 0); // 
    analogWrite(Motor_2_PWM, speed2); //
    break;
  }  
}  

void body_hardturn() {    // inner wheel spins at 1/3 rate of outer wheel
  switch (direction) {
  case 0:  // turn left
    turnservo.write(servo49);        // turn left
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, speed1); // 
    analogWrite(Motor_2_PWM, (speed2)/3); //
    break;
  case 1:  // turn right
    turnservo.write(servo131);        // turn right
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, (speed1)/3); // 
    analogWrite(Motor_2_PWM, speed2); //
    break;
  }
}  

void body_softturn() {    // inner wheel spins at 2/3 rate of outer wheel
  switch (direction) {
  case 0:  // turn left
    turnservo.write(servo71);        // turn left
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, speed1); // 
    analogWrite(Motor_2_PWM, 2*(speed2)/3); //
    break;
  case 1:  // turn right
    turnservo.write(servo109);        // turn right
//    delay(servo_turn);  // wait for servo to stop turning; better to change motor speed right away for this turn
    digitalWrite(Motor_1_Dir, LOW); // forward
    digitalWrite(Motor_2_Dir, LOW); // forward
    analogWrite(Motor_1_PWM, 2*(speed1)/3); // 
    analogWrite(Motor_2_PWM, speed2); //
    break;
  }
}  

void totalhalt() {
  digitalWrite(Motor_1_PWM, LOW);
  digitalWrite(Motor_1_Dir, LOW);
  digitalWrite(Motor_2_PWM, LOW);
  digitalWrite(Motor_2_Dir, LOW);
  beep();
}  

void beep() {
  for (int i=0; i<500; i++) {  // generate a 1KHz tone for 1/2 second
    digitalWrite(piezo, HIGH);
    delayMicroseconds(500);
    digitalWrite(piezo, LOW);
    delayMicroseconds(500);
  } 
}







